#define ID "FTWGEDfx v1.2.1"
/* #define DEBUG 1 /* Debug flag - uncomment to activate */
/* FTWGEDfx - a program to fix a GEDCOM file created by FTW

It relies very strongly on the fact that GEDCOM output by FTW 7.0
has a very defined and strict format. it must be generated with abbreviated
tags (with or without indent). Probably any character set will work.

				 Copyright 2000 Rick Parsons

					OPEN SOURCE FREEWARE

i.e. You are free to use and pass on the program or any part of its source (but
including this copyright notice) but not for gain or reward beyond the retail
cost of the media.

I am prepared to make an exception. If Broderbnd, Banner Blue, Parsons Technology,
The Learning Company, Genealogy.com or whatever they call themselves at the moment,
want to incorporate these changes into any future version of Family Tree Maker for
Windows, then they are quite welcome to as this would save everyone a lot of effort.

This program has only been tested with FTW Version 7.0. There is very little
error checking so it relies on unmolested source data.

v1,2,1 Recompiled with MinGW32
*/

#include "FTWGEDfx.h"

static int lno = 0;
logical ged_syntax(char *, int *, char *, char *, int *);

void _main(int argc, char *argv[])
{
	char line[GEDLINE], pline[GEDLINE], xref[10], tag[10], ptag[10];
	char * lp, *value, *np, *x1, *x2;
	int c, level, voffset;
	logical repo = FALSE, px = FALSE;
#ifdef DEBUG
	fprintf(stderr, "debug: mode active\n");
#endif
	while (TRUE)
	{
		/* read in each line from the source */
		for (lp = line; (c = getchar()) != '\n'; )
		{
			if (c == EOF)
				break;
			if ((lp - line) < (GEDLINE - 1))
				*(lp++) = (char)c;
		}

		if (c == EOF)
			break;

		lno++;

		if (lp == line)
			continue;	/* ignore empty lines */

		*lp = (char)NULL;	/* teminate the line */

		if (!ged_syntax(line, &level, xref, tag, &voffset))
			continue;
		value = line + voffset;

		/* now test for special cases that require editing
		There is a lot of checking that could be done here to make sure that
		the levels are right and xrefs are missing etc. */
		if (strcmp(tag, "ALIA") == 0)
		{
			printf("%d NICK ", level + 1);
			/* strip out surname delimiter characters */
			for (np = value; *np != (char)NULL; np++)
				if (*np != '/')
					putchar(*np);
			putchar('\n');
			continue; /* to put out preserved line next */
		}
		else if (px)
		{ /* flush the previous saved line */
			printf("%s\n", pline);
			px = FALSE;
		}

		if (strcmp(tag, "_MILT") == 0)
			printf("%d EVEN\n%d TYPE Military service\n", level, level + 1);
		else if (strcmp(tag, "_DEG") == 0)
			printf("%d EVEN\n%d TYPE Degree\n", level, level + 1);
		else if (strcmp(tag, "_MDCL") == 0)
			printf("%d EVEN\n%d TYPE Medical\n%d PLAC %s\n", level, level + 1,
				level + 1, value);
		else if (strcmp(tag, "DATE") == 0)
		{ /* now the value part is significant */
			if (strcmp(value, "UNKNOWN") == 0)
				printf("%d DATE (Unknown)\n", level);
			else if (strcmp(value, "PRIVATE") == 0)
				printf("%d DATE (Private)\n", level);
			else if (strncmp(value, "BEF.", 4) == 0)
				printf("%d DATE BEF %s\n", level, value + 5);
			else if (strncmp(value, "AFT.", 4) == 0)
				printf("%d DATE AFT %s\n", level, value + 5);
			else if (strncmp(value, "ABT.", 4) == 0)
				printf("%d DATE ABT %s\n", level, value + 5);
			else if (strncmp(value, "BET.", 4) == 0)
			{
				if ((strcmp(ptag, "OCCU") == 0) | (strcmp(ptag, "RESI") == 0) |
					(strcmp(ptag, "_MILT") == 0))
				{
					x1 = "FROM";
					x2 = "TO";
				}
				else
				{
					x1 = "BET";
					x2 = "AND";
				}
				/* find the hyphen in the date */
				for (np = value + 5; *np != '-'; np++)
					;
				*np = (char)NULL;

				printf("%d DATE %s %s%s%s\n", level, x1, value + 5, x2, np + 1);
			}
			else
         	printf("%d DATE %s\n", level, value);
		}
		/* Repository record should have an XREF to a level 0 REPO record */
		else if (strcmp(tag, "REPO") == 0)
		{
			printf("%d REPO @R1@\n", level);
			repo = TRUE;
		}
		/* Trailer record, insert the level 0 REPO record */
		else if (strcmp(tag, "TRLR") == 0)
		{
			if (repo)
				printf("0 @R1@ REPO\n");
			printf("0 TRLR\n");
		}
		else /* save the line for later */
		{
			strcpy(pline, line);
			px = TRUE;
		}
      /* save the last tag in case the date code needs it */
		strcpy(ptag, tag);
	}

	fprintf(stderr, "%s: PROGRAM ENDED\n", ID);

	_exit_(0);
}

logical ged_syntax(char * line, int * level, char * xref, char * tag, int * voffset)
{
	/* the format of all GEDCOM lines is

		1	a level number (int)
		2	an optional xref delimited by @@
		3	a record tag
		4	optional free form value to the end of the line
			(returned as an offset in line)
	*/

	char * cp, * levelp, * xrefp, * tagp;

#ifdef DEBUG
	fprintf(stderr, "debug: ged_syntax processing record %d <%s>\n", lno, line);
#endif

	/* first find the level number */
	/* remove leading spaces */
	for (cp = line ; *cp == ' '; cp++)
		;

	/* check for end of line again, just to make sure - blank line */
	if (*cp == (char)NULL)
		return FALSE;

	/* store the pointer into the correct place */
	levelp = cp;

	/* now search for end of field, while we are at it, check for digits */
	for ( ; *cp != ' ' && *cp != '@' && *cp != (char)NULL; cp++)
		if (!isdigit(*cp))
		{
			printf("%s: Error: GEDCOM level is not numeric on record %d\n%s\n",
				ID, lno, line);
			return FALSE;
		}

	*level = atoi(levelp);

	/* remove more spaces */
	for ( ; *cp == ' '; cp++)
		;

	/* check for end of line again, just to make sure */
	if (*cp == (char)NULL)
	{
		printf("%s: Error: GEDCOM incomplete record no %d\n%s\n", ID, lno, line);
		return FALSE;
	}

	/* we have seen the level, so it is now an ident or a tag */
	xrefp = xref;
	if (*cp == '@')
	{
		if (*level != 0)
		{
			printf("%s: Warning: GEDCOM xref not on level 0, record %d\n%s\n",
				ID, lno, line);
		}

		/* find end of xref (NO CHECK TO ENSURE XREF NOT TOO LONG) */
		for ( cp++; *cp != '@' && *cp != (char)NULL; cp++)
			*xrefp++ = *cp;

		/* check for end of line again, just to make sure */
		if (*cp == (char)NULL)
		{
			printf("%s: Error: GEDCOM incomplete ident on record %d\n%s\n",
				ID, lno, line);
			return FALSE;
		}

		/* skip over delimiter */
		cp++;
	}
	*xrefp = (char)NULL;

	/* remove more spaces */
	for ( ; *cp == ' '; cp++)
		;

	/* check for end of line again, just to make sure */
	if (*cp == (char)NULL)
	{
		printf("%s: Error: GEDCOM incomplete record no %d\n%s\n", ID, lno, line);
		return FALSE;
	}

	/* now must be tag */
	tagp = tag;
	/* now search for end of field (NO CHECK TO ENSURE TAG NOT TOO LONG) */
	for ( ; *cp != ' ' && *cp != (char)NULL; cp++)
		*tagp++ = *cp;
	*tagp = (char)NULL;

	/* remove spaces one last time */
	for ( ; *cp == ' '; cp++)
		;
	*voffset = cp - line;

	return TRUE;
}

