/* A wrapper to enable command line type programs to run in an easywin
environment */

#include <stddef.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <io.h>
#include <fcntl.h>
#include <sys\stat.h>
#include <ctype.h>
#include <dir.h>

int _main(int, char **);

void _redirect(const char *infile, const char *outfile)
{
#define STDIN 0
#define STDOUT 1
#define STDERR 2
	int fh;
	/* perform input redirection */
	if (infile)
	{
		if ((fh = open(infile, O_RDONLY, 0))>=0)
		{
			dup2(fh, STDIN);
			close(fh);
		}
		else
		{
			puts("wrapper: Open of STDIN redirected file failed\n");
			exit(12);
		}
	}

	/* perform output redirection */
	if (outfile)
	{
		if (*outfile == '>')
		{
			if (outfile[1] == '&')	/* redirect stderr as well */
			{
				fh = open(outfile + 2, O_CREAT | O_APPEND | O_WRONLY, S_IWRITE);
				outfile = NULL;		/* flag to do stderr as well */
			}
			else
				fh = open(outfile + 1, O_CREAT | O_APPEND | O_WRONLY, S_IWRITE);
		}
		else
		{
			if (*outfile == '&')		/* redirect stderr as well */
			{
				fh = creat(outfile + 1, S_IWRITE);
				outfile=NULL;		/* flag to do stderr as well */
			}
			else
				fh = creat(outfile, S_IWRITE);
		}
		
		if (fh >= 0)
		{
			dup2(fh, STDOUT);
			if (!outfile)		/* redirect stderr as well */
				dup2(fh, STDERR);
			close(fh);
		}
		else
		{
			puts("wrapper: Open of STDOUT/ERR redirected file failed\n");
			exit(12);
		}
	}
}

int main()
{
	char * infile=NULL, * outfile=NULL;
	char * args[100];	/* room for 100 arguments */
	char argt[1000];	/* room for 1000 characters */
	char * line;
	char ** argv = args;
	char cwd[128];

	if (getcwd(cwd, 128) == (char *)NULL)
		*cwd = (char)NULL;

	/* read the command line in from screen */
	fputs("Enter command ", stdout);
	if (*cwd != (char)NULL)
	{
		fputs("(file paths relative to ", stdout);
		fputs(cwd, stdout);
		fputs(") ", stdout);
	}
	putchar(':');
	line = argt;
	(void)gets(line);

	*argv++ = (char *)"";	/* ANSI requirement when program name unavailable */

	/* now interpret the command line puting values into the pointers */
	for (;;)
	{
		char c;

		while (*line == ' ' || *line == '\t')
			line++;
		if (!*line)
			break;	/* end of command line */
		switch(c = *line)
		{
			case '<':	/* stdin redirect */
				infile = ++line;
				c = '\0';
				break;
				
			case '>':	/* stdout redirect */
				outfile = ++line;
				c = '\0';
				break;

			case '\'':
			case '"':	/* quoted text */
				*argv++ = ++line;
				break;

			case '\\': /* continuation line */
				if (*(line + 1) == (char)NULL) /* only if end of line */
				{	(void) puts("Continue command line: ");
					(void) gets(line);
					continue;
				}
				/* else drop through to default */

			default:
				*argv++ = line;
				c = '\0';
				break;
		}
		while (*line && (c?*line!=c:(*line!=' '&&*line!='\t')))
			line++;
		if (*line)
			*line++='\0';

 //		/* Now work out wild cards if we a) havn't reached redirect
 //			b) if the arg is not quoted and c) if it contains * or ? */
 //		if (!outfile && !infile && !c && strpbrk(*(argv - 1), "*?"))
 //			/* decode wild string (ignore if fails) */
 //			if ((n = getfnl(*(argv - 1), nam, nam - names + 1000, 0)) > 0)
 //				/* make into a proper argument list */
 //				if (strbpl(argv - 1, 100, nam) == n)
 //				{	/* sort it for neatness */
 //					strsrt(argv - 1, n);
 //
 //					/* position argument pointer */
 //					argv += n - 1;
 //
 //					/* reposition buffer area for another one */
 //					nam = *(argv - 1) + strlen(*(argv - 1)) + 1;
 //				}
 //				else
 //				{	cputs("_setargv: work area overflow\r\n");
 //					_exit(1);
 //				}
 //			else
 //			{	cputs("_setargv: No match\r\n");
 //				_exit(1);
 //			}
	}

	_redirect(infile, outfile);

	*argv++ = NULL;		/* UNIX requirement */

	/* now call the real main program with these values */
	return _main(argv - args - 1, args);
}